<?php

final class Dm_Elementor_Extension
{
    const VERSION = '1.0.0';
	const MINIMUM_ELEMENTOR_VERSION = '3.0.11';
	const MINIMUM_PHP_VERSION = '7.2';
    const MINIMUM_DIGIMEMBER_VERSION = '3.000.166';

	private static $_instance = null;
	public static function instance()
    {
        if (is_null(self::$_instance)) {
            self::$_instance =  new self();
        }
        return self::$_instance;
    }

	public function __construct()
    {
        add_action('init', [$this, 'i18n']);
        add_action('plugins_loaded', [$this, 'init']);
    }
    
	public function init()
    {
        // Check if Elementor installed and activated
		if (!did_action('elementor/loaded') || !did_action('digimember/loaded')) {
			add_action('admin_notices', [$this, 'admin_notice_missing_main_plugin']);
			return;
		}
        // Check for required Elementor version
		if (!version_compare(ELEMENTOR_VERSION, self::MINIMUM_ELEMENTOR_VERSION, '>=')) {
			add_action('admin_notices', [$this, 'admin_notice_minimum_elementor_version']);
			return;
		}
        // Check for required DigiMember version
		if (!version_compare(DIGIMEMBER_VERSION, self::MINIMUM_DIGIMEMBER_VERSION, '>=')) {
			add_action('admin_notices', [$this, 'admin_notice_minimum_digimember_version']);
			return;
		}
        // Check for required PHP version
		if (version_compare(PHP_VERSION, self::MINIMUM_PHP_VERSION, '<')) {
			add_action('admin_notices', [$this, 'admin_notice_minimum_php_version']);
			return;
		}

        $this->i18n();
        $this->includes();
    }
    
    public function includes()
    {
        require_once dirname(__FILE__) . '/hooks.php';
    }

    public function i18n()
    {
        require_once dirname(__FILE__) . '/locale.php';
    }

    public function admin_notice_minimum_php_version()
    {
        if (isset($_GET['activate'])) unset($_GET['activate']);

		$message = sprintf(
			/* translators: 1: Plugin name 2: PHP 3: Required PHP version */
			esc_html__('"%1$s" benötigt "%2$s" in der Version %3$s oder höher.','digimember-elementor'),
			'<strong>' . esc_html__('DigiMember Elementor Erweiterung', 'digimember-elementor') . '</strong>',
			'<strong>' . esc_html__('PHP', 'digimember-elementor') . '</strong>',
			 self::MINIMUM_PHP_VERSION
		);

		printf('<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message);
    }

    public function admin_notice_minimum_digimember_version()
    {
        
        if ( isset( $_GET['activate'] ) ) unset( $_GET['activate'] );

		$message = sprintf(
			esc_html__('"%1$s" benötigt "%2$s" in der Version %3$s oder höher.', 'digimember-elementor'),
			'<strong>' . esc_html__('DigiMember Elementor Erweiterung', 'digimember-elementor') . '</strong>',
			'<strong>' . esc_html__('DigiMember', 'digimember-elementor') . '</strong>',
			 self::MINIMUM_DIGIMEMBER_VERSION
		);

		printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );
    }

    public function admin_notice_minimum_elementor_version()
    {
        
        if ( isset( $_GET['activate'] ) ) unset( $_GET['activate'] );

		$message = sprintf(
			esc_html__('"%1$s" benötigt "%2$s" in der Version %3$s oder höher.', 'digimember-elementor'),
			'<strong>' . esc_html__('DigiMember Elementor Erweiterung', 'digimember-elementor') . '</strong>',
			'<strong>' . esc_html__('Elementor', 'digimember-elementor') . '</strong>',
			 self::MINIMUM_ELEMENTOR_VERSION
		);

		printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );
    }
    
    public function admin_notice_missing_main_plugin()
    {
        if (isset($_GET['activate'])) unset($_GET['activate']);

		$message = sprintf(
			esc_html__('Für "%1$s" müssen "%2$s" und "%3$s" installiert und aktiviert sein.', 'digimember-elementor'),
			'<strong>' . esc_html__( 'DigiMember Elementor Erweiterung', 'digimember-elementor' ) . '</strong>',
			'<strong>' . esc_html__( 'Elementor', 'digimember-elementor' ) . '</strong>',
			'<strong>' . esc_html__( 'DigiMember', 'digimember-elementor' ) . '</strong>'
		);

		printf('<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message);
    }
}

Dm_Elementor_Extension::instance();
