<?php /** @noinspection PhpIncludeInspection */

use Elementor\Controls_Manager;
use Elementor\Core\Base\Document;
use Elementor\Elements_Manager;
use Elementor\Plugin;
use Elementor\Widgets_Manager;

$dm_elementor_should_replace_content = [];
add_action('digimember/access/accessType', function ($accessType, $post_type, $obj_or_id, $user_id) use (&$dm_elementor_should_replace_content) {
    $id = 0;
    if ($obj_or_id instanceof WP_Post) {
        $id = $obj_or_id->ID;
    } else if (is_numeric($obj_or_id)) {
        $id = (int)$obj_or_id;
    }
    if (!isset(Plugin::$instance->db) || !$id || !Plugin::$instance->db->is_built_with_elementor($id)) {
        return $accessType;
    }
    [$access_type, $wait_days, $product_id, $unlock_datetime] = $accessType;
    if ($access_type == DIGI_ACCESS_NONE) {
        $dm_elementor_should_replace_content[$id] = [
            'access_type' => $access_type,
            'wait_days' => $wait_days,
            'product_id' => $product_id,
            'unlock_datetime' => $unlock_datetime,
        ];
        $access_type = DIGI_ACCESS_FULL;
    }
    return [
        $access_type,
        $wait_days,
        $product_id,
        $unlock_datetime,
    ];
}, 10, 4);

add_action('elementor/frontend/the_content', function ($content) use (&$dm_elementor_should_replace_content) {
    $settings = Plugin::$instance->documents->get_current()->get_settings();
    $id = Plugin::$instance->documents->get_current()->get_main_id();
    if (isset($dm_elementor_should_replace_content[$id]) && is_array($dm_elementor_should_replace_content[$id])) {
        $wait_days = $dm_elementor_should_replace_content[$id]['wait_days'];
        $unlock_datetime = $dm_elementor_should_replace_content[$id]['unlock_datetime'];
        if (is_numeric($wait_days)  && isset($settings['digimember_wait_template'])) {
            $newContent = Plugin::$instance->frontend->get_builder_content($settings['digimember_wait_template']);
            
            dm_api()->load->helper('date');
            $wait_seconds = 86400 * $wait_days;
            $date         = ncore_formatDate(time() + $wait_seconds);
            $time         = '0:00';
            $in_days      = ncore_formatTimeSpan($wait_seconds, 'in');
    
            $find = ['[DAYS]', '[DATE]', '[TIME]', '[IN_DAYS]'];
            $repl = [$wait_days, $date, $time, $in_days];

            return str_replace($find, $repl, $newContent);
        } else if ($wait_days === false && isset($settings['digimember_locked_template']) && $settings['digimember_locked_template'] != "0") {
            return Plugin::$instance->frontend->get_builder_content($settings['digimember_locked_template']);
        }
    }

    return $content;
});

add_action('elementor/documents/register_controls', function (Document $instance) {
    if (!($instance instanceof Elementor\Core\DocumentTypes\Page) && !($instance instanceof Elementor\Core\DocumentTypes\Post)) {
        return;
    }

    $instance->start_controls_section(
        'digimember_settings',
        [
            'label' => _dmelem('DigiMember Einstellungen'),
            'tab' => Controls_Manager::TAB_SETTINGS,
        ]
    );
    $templates = Plugin::$instance->templates_manager->get_source('local')->get_items();
    $options = [0 => _dmelem('Ohne')];
    foreach ($templates as $template) {
        $options[$template['template_id']] = $template['title'];
    }
    $instance->add_control(
        'digimember_locked_template',
        [
            'label' => _dmelem('Gesperrt'),
            'type' => Controls_Manager::SELECT,
            'default' => $instance->get_main_post()->post_status,
            'options' => $options,
        ]
    );
    $instance->add_control(
        'digimember_wait_template',
        [
            'label' => _dmelem('Freischaltung'),
            'type' => Controls_Manager::SELECT,
            'description' => _dmelem('Verfügbare Shortcodes im Text: [DAYS], [DATE], [TIME], [IN_DAYS]'),
            'default' => $instance->get_main_post()->post_status,
            'options' => $options,
        ]
    );
    $instance->end_controls_section();
});

add_action('elementor/widgets/widgets_registered', function (Widgets_Manager $manager) {
    $widgets = [
        'login',
        'signup',
        'products',
    ];

    foreach ($widgets as $widget_filename) {
        require_once dirname(__FILE__) . '/modules/widgets/' . $widget_filename . '.php';
        $class_name = ucfirst(str_replace('-', '_', $widget_filename));
        $class_name = 'DigiMember\Elementor\Modules\Widgets\\' . $class_name;
        $manager->register_widget_type(new $class_name());
    }
});

add_action('elementor/elements/elements_registered', function () {
    $widgets = [
        'section',
        'column',
    ];
    foreach ($widgets as $widget_filename) {
        Plugin::$instance->elements_manager->unregister_element_type($widget_filename);

        require_once dirname(__FILE__) . '/modules/elements/' . $widget_filename . '.php';
        $class_name = ucfirst(str_replace('-', '_', $widget_filename));
        $class_name = 'DigiMember\Elementor\Modules\Elements\\' . $class_name;
        Plugin::$instance->elements_manager->register_element_type(new $class_name());
    }
});

add_action('elementor/elements/categories_registered', function (Elements_Manager $manager) {
    $manager->add_category('digimember', [
        'title' => 'DigiMember',
    ]);
});

add_action('elementor/editor/after_enqueue_styles', function () {
    wp_enqueue_style('dmelem_styles', plugins_url('assets/styles.css', __FILE__), false, '1.0.0', 'all');
});

add_action('elementor/frontend/after_enqueue_styles', function() {
    wp_enqueue_style('dmelem_frontend_styles', plugins_url('assets/frontend.css', __FILE__), false, '1.0.0', 'all');
});
