<?php

namespace DigiMember\Elementor\Modules\Elements;

use Elementor\Controls_Manager;
use Elementor\Plugin;

trait Ds_if_trait
{
    public function get_icon()
    {
        return 'icon-digimember';
    }

    protected function _register_controls()
    {
        parent::_register_controls();

        $dmProducts = $this->getDmProducts();

        $this->start_controls_section(
            'section_digimember',
            [
                'label' => 'DigiMember',
                'tab' => Controls_Manager::TAB_CONTENT,
            ]
        );

        $this->add_control(
            'ds_if',
            [
                'label' => _dmelem('Element schützen'),
                'type' => Controls_Manager::SWITCHER,
                'default' => '',
            ]
        );

        $this->add_control(
            'ds_if_has_product',
            [
                'label' => _dmelem('Hat Produkt'),
                'type' => Controls_Manager::SELECT2,
                'multiple' => true,
                'options' => $dmProducts,
                'condition' => [
                    'ds_if' => 'yes',
                ],
            ]
        );

        $this->add_control(
            'ds_if_has_not_product',
            [
                'label' => _dmelem('Hat Produkt nicht'),
                'type' => Controls_Manager::SELECT2,
                'multiple' => true,
                'options' => $dmProducts,
                'condition' => [
                    'ds_if' => 'yes',
                ],
            ]
        );

        $this->add_control(
            'ds_if_logged_in',
            [
                'label' => _dmelem('Ist eingeloggt'),
                'type' => Controls_Manager::SELECT,
                'default' => '-1',
                'options' => [
                    '-1' => '',
                    'no' => _dmelem('Nein'),
                    'yes' => _dmelem('Ja'),
                ],
                'condition' => [
                    'ds_if' => 'yes',
                ],
            ]
        );

        $this->add_control(
            'ds_if_mode',
            [
                'label' => _dmelem('Nur wenn'),
                'type' => Controls_Manager::SELECT,
                'default' => '-1',
                'options' => [
                    '-1' => '',
                    'else' => _dmelem('die vorherige Bedingung falsch war'),
                    'finally' => _dmelem('jede Bedingung bisher falsch war'),
                ],
                'condition' => [
                    'ds_if' => 'yes',
                ],
            ]
        );


        $this->end_controls_section();
    }

    /**
     * @return array
     */
    private function getDmProducts()
    {
        $dmProducts = digimember_listProducts();
        $arr = ncore_listToArray($dmProducts, 'id', 'name');
        ksort($arr);
        return $arr;
    }

    public function print_element()
    {
        if ($this->isProtected()) {
            return;
        }
        parent::print_element();
    }

    private function isProtected()
    {
        if (Plugin::$instance->editor->is_edit_mode()) {
            return false;
        }

        $settings = $this->get_settings_for_display();
        if (ncore_retrieve($settings, 'ds_if') != 'yes') {
            return false;
        }

        $hasProduct = ncore_retrieve($settings, 'ds_if_has_product', []);
        if (is_array($hasProduct)) {
            $hasProduct = implode(',', $hasProduct);
        }

        $hasNotProduct = ncore_retrieve($settings, 'ds_if_has_not_product', []);
        if (is_array($hasNotProduct)) {
            $hasNotProduct = implode(',', $hasNotProduct);
        }

        $loggedIn = ncore_retrieve($settings, 'ds_if_logged_in', '');
        $loggedIn = $loggedIn == '-1' ? '' : $loggedIn;
        $mode = ncore_retrieve($settings, 'ds_if_mode', '');
        $mode = $mode == '-1' ? '' : $mode;

        $shortcodeController = dm_api()->load->controller('shortcode');
        $shortcodeResult = $shortcodeController->shortcodeIf([
            'has_product' => $hasProduct,
            'has_not_product' => $hasNotProduct,
            'logged_in' => $loggedIn,
            'mode' => $mode,
        ], 'hasPassed');

        return $shortcodeResult == 'hasPassed' ? false : true;
    }
}