<?php

namespace DigiMember\Elementor\Modules\Widgets;

use digimember_ShortCodeController;
use digimember_UserLoginFormController;
use Elementor\Core\Kits\Documents\Tabs\Global_Colors;
use Elementor\Core\Kits\Documents\Tabs\Global_Typography;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Typography;
use Elementor\Plugin;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use ncore_AjaxLib;

/**
 * Class Login
 * @package DigiMember\Elementor\Modules\Widgets
 */
class Login extends Widget_Base
{
    public function get_name()
    {
        return 'dm_login';
    }

    public function get_title()
    {
        return _dmelem('Login Formular');
    }

    public function get_icon()
    {
        return 'icon-digimember';
    }

    public function get_keywords()
    {
        return ['digimember', 'login', 'user', 'form'];
    }

    public function get_categories()
    {
        return ['digimember'];
    }

    protected function _register_controls()
    {
        $this->_register_general_controls();
        $this->_register_form_field_controls();
        $this->_register_button_controls();
        $this->_register_label_controls();
        $this->_register_style_controls();
    }

    protected function _register_style_controls()
    {
        $this->start_controls_section(
            'section_style',
            [
                'label' => _dmelem('Formular'),
                'tab' => Controls_Manager::TAB_STYLE,
            ]
        );

        $this->add_control(
            'row_gap',
            [
                'label' => _dmelem('Zeilenabstand'),
                'type' => Controls_Manager::SLIDER,
                'default' => [
                    'size' => '10',
                ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 60,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .elementor-field-group' => 'margin-bottom: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .elementor-form-fields-wrapper' => 'margin-bottom: -{{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'links_color',
            [
                'label' => _dmelem('Linkfarbe'),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .elementor-field-group a' => 'color: {{VALUE}};',
                ],
                'global' => [
                    'default' => Global_Colors::COLOR_TEXT,
                ],
            ]
        );

        $this->add_control(
            'links_hover_color',
            [
                'label' => _dmelem('Linkfarbe schwebend'),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .elementor-field-group a:hover' => 'color: {{VALUE}};',
                ],
                'global' => [
                    'default' => Global_Colors::COLOR_ACCENT,
                ],
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'section_style_labels',
            [
                'label' => _dmelem('Labels'),
                'tab' => Controls_Manager::TAB_STYLE,
                'condition' => [
                    'show_labels!' => '',
                ],
            ]
        );

        $this->add_control(
            'label_spacing',
            [
                'label' => _dmelem('Abstand'),
                'type' => Controls_Manager::SLIDER,
                'default' => [
                    'size' => '0',
                ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 60,
                    ],
                ],
                'selectors' => [
                    'body {{WRAPPER}} .elementor-field-group > label' => 'padding-bottom: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'label_color',
            [
                'label' => _dmelem('Textfarbe'),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .elementor-form-fields-wrapper label' => 'color: {{VALUE}};',
                ],
                'global' => [
                    'default' => Global_Colors::COLOR_TEXT,
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'label_typography',
                'selector' => '{{WRAPPER}} .elementor-form-fields-wrapper label',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_TEXT,
                ],
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'section_field_style',
            [
                'label' => _dmelem('Felder'),
                'tab' => Controls_Manager::TAB_STYLE,
            ]
        );

        $this->add_control(
            'field_text_color',
            [
                'label' => __('Textfarbe'),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .elementor-field-group .elementor-field' => 'color: {{VALUE}};',
                ],
                'global' => [
                    'default' => Global_Colors::COLOR_TEXT,
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'field_typography',
                'selector' => '{{WRAPPER}} .elementor-field-group .elementor-field, {{WRAPPER}} .elementor-field-subgroup label',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_TEXT,
                ],
            ]
        );

        $this->add_control(
            'field_background_color',
            [
                'label' => _dmelem('Hintergrundfarbe'),
                'type' => Controls_Manager::COLOR,
                'default' => '#ffffff',
                'selectors' => [
                    '{{WRAPPER}} .elementor-field-group .elementor-field:not(.elementor-select-wrapper)' => 'background-color: {{VALUE}};',
                    '{{WRAPPER}} .elementor-field-group .elementor-select-wrapper select' => 'background-color: {{VALUE}};',
                ],
                'separator' => 'before',
            ]
        );

        $this->add_control(
            'field_border_color',
            [
                'label' => _dmelem('Rahmenfarbe'),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .elementor-field-group .elementor-field:not(.elementor-select-wrapper)' => 'border-color: {{VALUE}};',
                    '{{WRAPPER}} .elementor-field-group .elementor-select-wrapper select' => 'border-color: {{VALUE}};',
                    '{{WRAPPER}} .elementor-field-group .elementor-select-wrapper::before' => 'color: {{VALUE}};',
                ],
                'separator' => 'before',
            ]
        );

        $this->add_control(
            'field_border_width',
            [
                'label' => _dmelem('Rahmenbreite'),
                'type' => Controls_Manager::DIMENSIONS,
                'placeholder' => '1',
                'size_units' => ['px'],
                'selectors' => [
                    '{{WRAPPER}} .elementor-field-group .elementor-field:not(.elementor-select-wrapper)' => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    '{{WRAPPER}} .elementor-field-group .elementor-select-wrapper select' => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'field_border_radius',
            [
                'label' => _dmelem('Rahmenrundung'),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors' => [
                    '{{WRAPPER}} .elementor-field-group .elementor-field:not(.elementor-select-wrapper)' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    '{{WRAPPER}} .elementor-field-group .elementor-select-wrapper select' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'section_button_style',
            [
                'label' => _dmelem('Button'),
                'tab' => Controls_Manager::TAB_STYLE,
            ]
        );

        $this->start_controls_tabs('tabs_button_style');

        $this->start_controls_tab(
            'tab_button_normal',
            [
                'label' => _dmelem('Normal'),
            ]
        );

        $this->add_control(
            'button_text_color',
            [
                'label' => _dmelem('Textfarbe'),
                'type' => Controls_Manager::COLOR,
                'default' => '',
                'selectors' => [
                    '{{WRAPPER}} .elementor-button' => 'color: {{VALUE}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'button_typography',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_ACCENT,
                ],
                'selector' => '{{WRAPPER}} .elementor-button',
            ]
        );

        $this->add_control(
            'button_background_color',
            [
                'label' => _dmelem('Hintergrundfarbe'),
                'type' => Controls_Manager::COLOR,
                'global' => [
                    'default' => Global_Colors::COLOR_ACCENT,
                ],
                'selectors' => [
                    '{{WRAPPER}} .elementor-button' => 'background-color: {{VALUE}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(), [
                'name' => 'button_border',
                'selector' => '{{WRAPPER}} .elementor-button',
                'separator' => 'before',
            ]
        );

        $this->add_control(
            'button_border_radius',
            [
                'label' => _dmelem('Rahmenrundung'),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors' => [
                    '{{WRAPPER}} .elementor-button' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'button_text_padding',
            [
                'label' => _dmelem('Textabstand'),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', 'em', '%'],
                'selectors' => [
                    '{{WRAPPER}} .elementor-button' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->end_controls_tab();

        $this->start_controls_tab(
            'tab_button_hover',
            [
                'label' => _dmelem('Schwebend'),
            ]
        );

        $this->add_control(
            'button_hover_color',
            [
                'label' => _dmelem('Textfarbe'),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .elementor-button:hover' => 'color: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'button_background_hover_color',
            [
                'label' => _dmelem('Hintergrundfarbe'),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .elementor-button:hover' => 'background-color: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'button_hover_border_color',
            [
                'label' => _dmelem('Rahmenfarbe'),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .elementor-button:hover' => 'border-color: {{VALUE}};',
                ],
                'condition' => [
                    'button_border_border!' => '',
                ],
            ]
        );

        $this->add_control(
            'button_hover_animation',
            [
                'label' => _dmelem('Effekt'),
                'type' => Controls_Manager::HOVER_ANIMATION,
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();

        $this->end_controls_section();
    }

    protected function _register_button_controls()
    {
        $this->start_controls_section(
            'section_login_button',
            [
                'label' => _dmelem('Button'),
            ]
        );

        $this->add_control(
            'button_text',
            [
                'label' => _dmelem('Text'),
                'type' => Controls_Manager::TEXT,
                'default' => _dmelem('Anmelden'),
            ]
        );

        $this->add_control(
            'button_size',
            [
                'label' => __('Size', 'elementor-pro'),
                'type' => Controls_Manager::SELECT,
                'options' => [
                    'xs' => __('Extra Small', 'elementor-pro'),
                    'sm' => __('Small', 'elementor-pro'),
                    'md' => __('Medium', 'elementor-pro'),
                    'lg' => __('Large', 'elementor-pro'),
                    'xl' => __('Extra Large', 'elementor-pro'),
                ],
                'default' => 'sm',
            ]
        );

        $this->add_responsive_control(
            'align',
            [
                'label' => __('Alignment', 'elementor-pro'),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'start' => [
                        'title' => __('Left', 'elementor-pro'),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => __('Center', 'elementor-pro'),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'end' => [
                        'title' => __('Right', 'elementor-pro'),
                        'icon' => 'eicon-text-align-right',
                    ],
                    'stretch' => [
                        'title' => __('Justified', 'elementor-pro'),
                        'icon' => 'eicon-text-align-justify',
                    ],
                ],
                'prefix_class' => 'elementor%s-button-align-',
                'default' => '',
            ]
        );

        $this->end_controls_section();
    }

    protected function _register_form_field_controls()
    {
        $this->start_controls_section(
            'section_login_form_fields',
            [
                'label' => _dmelem('Felder'),
            ]
        );

        $this->add_control(
            'show_labels',
            [
                'label' => _dmelem('Labels anzeigen'),
                'type' => Controls_Manager::SWITCHER,
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'input_size',
            [
                'label' => __('Input Size', 'elementor-pro'),
                'type' => Controls_Manager::SELECT,
                'options' => [
                    'xs' => __('Extra Small', 'elementor-pro'),
                    'sm' => __('Small', 'elementor-pro'),
                    'md' => __('Medium', 'elementor-pro'),
                    'lg' => __('Large', 'elementor-pro'),
                    'xl' => __('Extra Large', 'elementor-pro'),
                ],
                'default' => 'sm',
            ]
        );

        $this->end_controls_section();
    }

    protected function _register_general_controls()
    {
        $this->start_controls_section(
            'section_login_general',
            [
                'label' => _dmelem('Allgemein'),
                'active' => true,
            ]
        );

        $this->add_control(
            'hidden_if_logged_in',
            [
                'label' => _dmelem('Versteckt falls, eingeloggt'),
                'type' => Controls_Manager::SWITCHER,
            ]
        );

        $this->add_control(
            'stay_on_same_page',
            [
                'label' => _dmelem('Auf der aktuellen Seite bleiben'),
                'type' => Controls_Manager::SWITCHER,
            ]
        );

        $this->add_control(
            'redirect_if_logged_in',
            [
                'label' => _dmelem('Weiterleiten falls eingeloggt'),
                'type' => Controls_Manager::SWITCHER,
            ]
        );

        $this->add_control(
            'show_lost_password',
            [
                'label' => _dmelem('Passwort vergessen anzeigen'),
                'type' => Controls_Manager::SWITCHER,
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'show_remember',
            [
                'label' => _dmelem('Eingeloggt bleiben anzeigen'),
                'type' => Controls_Manager::SWITCHER,
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'redirect_url',
            [
                'label' => _dmelem('Weiterleitungs-Url'),
                'type' => Controls_Manager::URL,
                'options' => false,
                'separator' => false,
                'dynamic' => [
                    'active' => true,
                ],
                'condition' => [
                    'stay_on_same_page' => '',
                ],
            ]
        );

        $this->add_control(
            'logout_url',
            [
                'label' => _dmelem('Weiterleitungs-Url nach dem Abmelden'),
                'type' => Controls_Manager::URL,
                'options' => false,
                'separator' => false,
                'dynamic' => [
                    'active' => true,
                ],
                'condition' => [
                    'hidden_if_logged_in' => '',
                    'redirect_if_logged_in' => '',
                ],
            ]
        );
        $this->end_controls_section();
    }

    protected function _register_label_controls()
    {
        $this->start_controls_section(
            'section_login_content',
            [
                'label' => _dmelem('Label'),
                'active' => false,
            ]
        );

        $this->add_control(
            'username_label',
            [
                'label' => _dmelem('Benutzername Label'),
                'type' => Controls_Manager::TEXT,
                'default' => _dmelem('Benutzername'),
            ]
        );

        $this->add_control(
            'username_placeholder',
            [
                'label' => _dmelem('Benutzername Platzhalter'),
                'type' => Controls_Manager::TEXT,
                'default' => _dmelem('Benutzername / E-Mail Adresse'),
            ]
        );

        $this->add_control(
            'password_label',
            [
                'label' => _dmelem('Passwort Label'),
                'type' => Controls_Manager::TEXT,
                'default' => _dmelem('Passwort'),
            ]
        );

        $this->add_control(
            'password_placeholder',
            [
                'label' => _dmelem('Passwort Platzhalter'),
                'type' => Controls_Manager::TEXT,
                'default' => _dmelem('Dein Passwort'),
            ]
        );

        $this->add_control(
            'remember_label',
            [
                'label' => _dmelem('Eingeloggt bleiben Label'),
                'type' => Controls_Manager::TEXT,
                'default' => _dmelem('Eingeloggt bleiben'),
            ]
        );

        $this->add_control(
            'lost_password_label',
            [
                'label' => _dmelem('Passwort vergessen Label'),
                'type' => Controls_Manager::TEXT,
                'default' => _dmelem('Passwort vergessen?'),
            ]
        );

        $this->end_controls_section();
    }

    private function form_fields_render_attributes()
    {
        $settings = $this->get_settings();

        if (!empty($settings['button_size'])) {
            $this->add_render_attribute('button', 'class', 'elementor-size-' . $settings['button_size']);
        }

        if ($settings['button_hover_animation']) {
            $this->add_render_attribute('button', 'class', 'elementor-animation-' . $settings['button_hover_animation']);
        }

        $username_id = ncore_id();
        $password_id = ncore_id();

        $this->add_render_attribute(
            [
                'wrapper' => [
                    'class' => [
                        'elementor-form-fields-wrapper',
                    ],
                ],
                'form' => [
                    'class' => [
                        'elementor-form'
                    ],
                    'method' => 'POST',
                    'id' => ncore_id(),
                ],
                'field-group' => [
                    'class' => [
                        'elementor-field-type-text',
                        'elementor-field-group',
                        'elementor-column',
                        'elementor-col-100',
                    ],
                ],
                'submit-group' => [
                    'class' => [
                        'elementor-field-group',
                        'elementor-column',
                        'elementor-field-type-submit',
                        'elementor-col-100',
                    ],
                ],
                'button' => [
                    'class' => [
                        'elementor-button',
                    ],
                    'id' => ncore_id(),
                    'name' => 'wp-submit',
                    'type' => 'submit',
                ],
                'username_label' => [
                    'for' => $username_id,
                    'class' => 'elementor-field-label',
                ],
                'username_input' => [
                    'type' => 'text',
                    'name' => 'username',
                    'id' => $username_id,
                    'placeholder' => $settings['username_placeholder'],
                    'class' => [
                        'elementor-field',
                        'elementor-field-textual',
                        'elementor-size-' . $settings['input_size'],
                    ],
                ],
                'password_label' => [
                    'for' => $password_id,
                    'class' => 'elementor-field-label',
                ],
                'password_input' => [
                    'type' => 'password',
                    'name' => 'password',
                    'id' => $password_id,
                    'placeholder' => $settings['password_placeholder'],
                    'class' => [
                        'elementor-field',
                        'elementor-field-textual',
                        'elementor-size-' . $settings['input_size'],
                    ],
                ],
                'remember_input' => [
                    'name' => 'remember',
                    'type' => 'checkbox',
                    'class' => 'ds-login-value-checkbox',
                    'id' => ncore_id(),
                    'value' => '1',
                ],
                'lost_password_link' => [
                    'id' => ncore_id(),
                ],
                'error' => [
                    'id' => ncore_id(),
                ]
            ]
        );

        if (!$settings['show_labels']) {
            $this->add_render_attribute('label', 'class', 'elementor-screen-only');
        }

        $this->add_render_attribute('field-group', 'class', 'elementor-field-required')
            ->add_render_attribute('input', 'required', true)
            ->add_render_attribute('input', 'aria-required', 'true');

    }

    protected function render()
    {
        $settings = $this->get_settings_for_display();

        if (ncore_isLoggedIn() && !Plugin::$instance->editor->is_edit_mode() && !Plugin::$instance->documents->get_current()->is_saving()) {
            $hidden = ncore_retrieve($settings, 'hidden_if_logged_in');
            if ($hidden != 'yes') {
                $this->_render_logged_in();
            }
            $shouldRedirect = ncore_retrieve($settings, 'redirect_if_logged_in');
            $url = $this->redirectUrl();
            if ($shouldRedirect == 'yes' && $url) {
                ncore_redirect($url);
            }
            return;
        }

        $this->form_fields_render_attributes();
        ?>
        <form <?php echo $this->get_render_attribute_string('form'); ?>>
            <div <?php echo $this->get_render_attribute_string('wrapper'); ?>>
                <div <?php echo $this->get_render_attribute_string('error'); ?>></div>
                <div <?php echo $this->get_render_attribute_string('field-group'); ?>>
                    <?php
                    if ($settings['show_labels']) {
                        echo '<label ' . $this->get_render_attribute_string('username_label') . '>' . $settings['username_label'] . '</label>';
                    }
                    echo '<input ' . $this->get_render_attribute_string('username_input') . '>';
                    ?>
                </div>
                <div <?php echo $this->get_render_attribute_string('field-group'); ?>>
                    <?php
                    if ($settings['show_labels']) {
                        echo '<label ' . $this->get_render_attribute_string('password_label') . '>' . $settings['password_label'] . '</label>';
                    }
                    echo '<input size="1" ' . $this->get_render_attribute_string('password_input') . '>';
                    ?>
                </div>
                <?php
                if ($settings['show_remember'] || $settings['show_lost_password']) {
                    ?>
                    <div class="elementor-field-group"
                         style="display: flex; width: 100%; align-items: center; justify-content: space-between;">
                        <?php
                        if ($settings['show_remember'] == 'yes') {
                            ?>
                            <div class="ds-login-remember">
                                <input <?php echo $this->get_render_attribute_string('remember_input'); ?> />
                                <label for="ds-login-remember"> <?php echo $settings['remember_label'] ?> </label>
                            </div>
                            <?php
                        }
                        if ($settings['show_lost_password'] == 'yes') {
                            ?>
                            <a <?php echo $this->get_render_attribute_string('lost_password_link'); ?> href="#">
                                <?php echo $settings['lost_password_label']; ?>
                            </a>
                            <?php
                        }
                        ?>
                    </div>
                    <?php
                }
                ?>
                <div <?php echo $this->get_render_attribute_string('submit-group'); ?>>
                    <button <?php echo $this->get_render_attribute_string('button'); ?>>
                        <?php
                        if (!empty($settings['button_text'])) {
                            ?>
                            <span class="elementor-button-text"><?php echo $settings['button_text']; ?></span>
                            <?php
                        }
                        ?>
                    </button>
                </div>
            </div>
        </form>
        <?php
        if (!Plugin::$instance->editor->is_edit_mode()) {
            ?>
            <script>
                (function ($) {
                    var $container = $('<?php echo sprintf('form#%s', $this->get_render_attributes('form')['id'][0]); ?>');
                    var $forgotPasswordButton = $container.find('<?php echo sprintf('a#%s', $this->get_render_attributes('lost_password_link')['id'][0]); ?>');
                    $forgotPasswordButton.on('click', function (event) {
                        event.preventDefault();

                        <?php echo $this->renderForgotPasswordDialog(); ?>
                    });

                    var $loginButton = $container.find('<?php echo sprintf('button#%s', $this->get_render_attributes('button')['id'][0]); ?>');
                    $('<div style="width: 100%;" id="%%ERROR_DIV_ID%%"></div>').insertBefore($loginButton);
                    $loginButton
                        .attr('type', 'submit')
                        .on('click', function (event) {
                            event.preventDefault();
                            var username = encodeURIComponent($container.find('<?php echo sprintf('input#%s', $this->get_render_attributes('username_input')['id'][0]); ?>').val());
                            var password = encodeURIComponent($container.find('<?php echo sprintf('input#%s', $this->get_render_attributes('password_input')['id'][0]); ?>').val());
                            var remember = $container.find('<?php echo sprintf('input#%s', $this->get_render_attributes('remember_input')['id'][0]); ?>').prop('checked') ? '1' : '0';

                            var data = {
                                'username': username,
                                'password': password,
                                'remember': remember,
                                'errordiv': '<?php echo $this->get_render_attributes('error')['id'][0]; ?>'
                            };

                            <?php echo $this->renderAjaxLoginJavascript(); ?>
                        });
                })(typeof ncoreJQ !== 'undefined' ? ncoreJQ : jQuery);
            </script>
            <?php
        }
    }

    private function _render_logged_in()
    {
        $settings = $this->get_settings_for_display();
        /** @var digimember_ShortCodeController $lib */
        $lib = dm_api()->load->controller('shortcode');
        $url = ncore_retrieve(ncore_retrieve($settings, 'logout_url', []), 'url');
        $opts = $url ? ['url' => $url] : [];

        echo $lib->shortcodeLoginInfo($opts);
    }


    /**
     * Render Login Form output in the editor.
     *
     * Written as a Backbone JavaScript template and used to generate the live preview.
     *
     * @since  2.9.0
     * @access protected
     */
    protected function content_template()
    {
        ?>
        <div class="elementor-form">
            <div class="elementor-form-fields-wrapper">
                <div class="elementor-field-group elementor-column elementor-col-100 elementor-field-type-text">
                    <# if (settings.show_labels) { #>
                    <label class="elementor-field-label" for="ds-login-username">
                        {{ settings.username_label }}
                    </label>
                    <# } #>
                    <input type="text" name="username" id="ds-login-username"
                           placeholder="{{ settings.username_placeholder }}"
                           class="elementor-field elementor-field-textual elementor-size-{{ settings.input_size }}"/>
                </div>
                <div class="elementor-field-group elementor-column elementor-col-100 elementor-field-type-text">
                    <# if (settings.show_labels) { #>
                    <label class="elementor-field-label" for="ds-login-password">
                        {{ settings.password_label }}
                    </label>
                    <# } #>
                    <input type="password" name="password" id="ds-login-password"
                           placeholder="{{ settings.password_placeholder }}"
                           class="elementor-field elementor-field-textual elementor-size-{{ settings.input_size }}"/>
                </div>
                <# if (settings.show_remember || settings.show_lost_password) { #>
                <div class="elementor-field-group"
                     style="display: flex; width: 100%; align-items: center; justify-content: space-between;">
                    <# if (settings.show_remember) { #>
                    <div class="ds-login-remember">
                        <input name="remember" type="checkbox" class="ds-login-value-checkbox"
                               id="ds-login-remember" value="1"/>
                        <label for="ds-login-remember"> {{ settings.remember_label }} </label>
                    </div>
                    <# } #>
                    <# if (settings.show_lost_password) { #>
                    <a href="#" id="ds-login-link-lost-password">
                        {{ settings.lost_password_label }}
                    </a>
                    <# } #>
                </div>
                <# } #>
                <div class="elementor-field-group elementor-column elementor-field-type-submit elementor-col-100">
                    <button type="submit" class="elementor-button elementor-size-{{ settings.button_size }}">
                        <span class="elementor-button-text">{{ settings.button_text }}</span>
                    </button>
                </div>
            </div>
        </div>
        <?php
    }

    /**
     * @return string
     */
    private function renderAjaxLoginJavascript()
    {
        /** @var digimember_UserLoginFormController $userFormController */
        $userFormController = dm_api()->load->controller('user/login_form');

        $args = [];
        $redirect_url = $this->redirectUrl();
        if ($redirect_url) {
            $args['redirect_url'] = $redirect_url;
        }
        $args['current_url'] = ncore_currentUrl();

        return $userFormController->renderAjaxJs('login', $args, 'data');
    }

    /**
     * @return string
     */
    private function redirectUrl()
    {
        $settings = $this->get_settings_for_display();
        $url = ncore_retrieve(ncore_retrieve($settings, 'redirect_url', []), 'url');

        if (ncore_retrieve($settings, 'stay_on_same_page') == 'yes') {
            $url = ncore_currentUrl();
        }

        return $url;
    }

    private function renderForgotPasswordDialog()
    {
        $meta = [
            'type' => 'form',
            'ajax_dlg_id' => 'ajax_forgotton_pw_dlg',
            'cb_controller' => 'user/login_form',
            'message' => _dgyou('Enter your email address. Only after you clicked the link in the confirmation email, we will create a new password for you.'),
            'title' => _dgyou('Set new password'),
            'width' => '500px',
            'form_sections' => [],
            'form_inputs' => [
                [
                    'name' => 'email',
                    'type' => 'text',
                    'label' => _dgyou('Email'),
                    'label_css' => 'ncore_texttoken',
                    'rules' => 'defaults|email',
                    'full_width' => true,
                ],
                [
                    'name' => 'redir_url',
                    'type' => 'hidden',
                    'default' => ncore_currentUrl(),
                ],
            ],
        ];

        /** @var ncore_AjaxLib $ajax */
        $ajax = dm_api()->load->library('ajax');

        $dialog = $ajax->dialog($meta);

        $javascript = $dialog->showDialogJs();

        return $javascript . '; return false;';
    }
}
