<?php

namespace DigiMember\Elementor\Modules\Widgets;

use digimember_UserProductData;
use digimember_ProductData;
use Elementor\Plugin;
use Elementor\Widget_Base;
use Elementor\Repeater;
use Elementor\Controls_Manager;
use Elementor\Core\Kits\Documents\Tabs\Global_Colors;
use Elementor\Core\Kits\Documents\Tabs\Global_Typography;
use Elementor\Group_Control_Image_Size;
use Elementor\Group_Control_Typography;

/**
 * Class Products
 * @package DigiMember\Elementor\Modules\Widgets
 */
class Products extends Widget_Base
{

    public function get_name()
    {
        return 'dm_products';
    }

    public function get_title()
    {
        return _dmelem('Produktliste');
    }

    public function get_icon()
    {
        return 'icon-digimember';
    }

    public function get_keywords()
    {
        return ['digimember', 'products'];
    }

    public function _register_controls()
    {
        $this->start_controls_section(
            'section_list',
            [
                'label' => _dmelem('Produktliste'),
                'tab' => Controls_Manager::TAB_CONTENT,
            ]
        );

        $this->add_control(
            'filter',
            [
                'label' => _dmelem('Produktfilter'),
                'type' => Controls_Manager::SELECT,
                'default' => 'user_has_product',
                'options' => [
                    'all' => _dmelem('Alle'),
                    'user_has_product' => _dmelem('User hat Produkt'),
                    'user_not_has_product' => _dmelem('User hat Produkt nicht'),
                ],
            ]
        );

        $repeater = new Repeater();

        $repeater->add_control(
            'id',
            [
                'type' => Controls_Manager::HIDDEN,
            ]
        );

        $repeater->add_control(
            'name',
            [
                'type' => Controls_Manager::HIDDEN,
            ]
        );

        $repeater->add_control(
            'visible',
            [
                'label' => _dmelem('Sichtbar'),
                'type' => Controls_Manager::SWITCHER,
                'default' => 'yes',
            ]
        );

        $repeater->add_control(
            'description',
            [
                'label' => _dmelem('Beschreibung'),
                'type' => Controls_Manager::TEXTAREA,
                'default' => '',
            ]
        );

        $repeater->add_control(
            'image',
            [
                'label' => _dmelem('Produktbild'),
                'type' => Controls_Manager::MEDIA,
                'default' => [],
            ]
        );

        $repeater->add_control(
            'link_enabled',
            [
                'label' => _dmelem('Shortcode Link verwenden'),
                'type' => Controls_Manager::SWITCHER,
                'default' => 'yes',
            ]
        );

        $repeater->add_control(
            'link_custom',
            [
                'label' => _dmelem('Eigener Link'),
                'type' => Controls_Manager::URL,
                'default' => ['url' => '#'],
                'dynamic' => [
                    'active' => true,
                ],
                'condition' => [
                    'link_enabled' => '',
                ],
            ]
        );

        $this->add_control(
            'products',
            [
                'label' => _dmelem('Produkte'),
                'type' => Controls_Manager::REPEATER,
                'fields' => $repeater->get_controls(),
                'item_actions' => [
                    'add' => false,
                    'duplicate' => false,
                    'remove' => false,
                ],
                'default' => array_map(function ($product) {
                    return (array)$product;
                }, digimember_listProducts()),
                'title_field' => '{{{ name }}}',
            ]
        );

        $this->add_control(
            'empty_list',
            [
                'label' => _dmelem('Hinweis bei leerer Liste'),
                'type' => Controls_Manager::WYSIWYG,
                'default' => _dmelem('Du hast noch keine Produkte'),
                'label_block' => true,
                'separator' => 'before',
            ]
        );

        $this->end_controls_section();

        $this->_register_style_controls();
    }

    private function _register_style_controls()
    {
        $this->start_controls_section(
            'section_products_style',
            [
                'label' => _dmelem('Produktliste'),
                'tab' => Controls_Manager::TAB_STYLE,
            ]
        );

        $this->add_control(
            'name__heading',
            [
                'label' => _dmelem('Produktname'),
                'type' => Controls_Manager::HEADING,
            ]
        );

        $this->add_control(
            'name_color',
            [
                'label' => _dmelem('Farbe'),
                'type' => Controls_Manager::COLOR,
                'global' => [
                    'default' => Global_Colors::COLOR_PRIMARY,
                ],
                'selectors' => [
                    '{{WRAPPER}} ul.dm-products > li > a > div > span' => 'color: {{VALUE}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'name_typography',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_PRIMARY,
                ],
                'selector' => '{{WRAPPER}} ul.dm-products > li > a > div > span',
            ]
        );

        $this->add_control(
            'description__heading',
            [
                'label' => _dmelem('Beschreibung'),
                'type' => Controls_Manager::HEADING,
                'separator' => 'before',
            ]
        );

        $this->add_control(
            'description_color',
            [
                'label' => _dmelem('Farbe'),
                'type' => Controls_Manager::COLOR,
                'global' => [
                    'default' => Global_Colors::COLOR_TEXT,
                ],
                'selectors' => [
                    '{{WRAPPER}} ul.dm-products > li > a > div > p' => 'color: {{VALUE}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'description_typography',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_TEXT,
                ],
                'selector' => '{{WRAPPER}} ul.dm-products > li > a > div > p',
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'section_image_style',
            [
                'label' => _dmelem('Bild'),
                'tab' => Controls_Manager::TAB_STYLE,
                'show_label' => false,
            ]
        );

        $this->add_group_control(
            Group_Control_Image_Size::get_type(),
            [
                'name' => 'image_size',
                'default' => 'thumbnail',
            ]
        );

        $this->add_control(
            'border_radius',
            [
                'label' => _dmelem('Randrundung'),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors' => [
                    '{{WRAPPER}} ul.dm-products > li > a > img' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'image_spacing',
            [
                'label' => _dmelem('Abstände'),
                'type' => Controls_Manager::SLIDER,
                'range' => [
                    'px' => [
                        'max' => 50,
                    ],
                ],
                'selectors' => [
                    'body.rtl {{WRAPPER}} ul.dm-products > li > a > img' => 'padding-left: calc({{SIZE}}{{UNIT}}/2);',
                    'body.rtl {{WRAPPER}} ul.dm-products:not(.no-image) > li > a > span.no-image' => 'padding-left: calc({{SIZE}}{{UNIT}}/2);',
                    'body.rtl {{WRAPPER}} ul.dm-products > li > a > img + div' => 'padding-right: calc({{SIZE}}{{UNIT}}/2);',
                    'body.rtl {{WRAPPER}} ul.dm-products:not(.no-image) > li > a > span.no-image + div' => 'padding-right: calc({{SIZE}}{{UNIT}}/2);',
                    'body:not(.rtl) {{WRAPPER}} ul.dm-products > li > a > img' => 'padding-right: calc({{SIZE}}{{UNIT}}/2);',
                    'body:not(.rtl) {{WRAPPER}} ul.dm-products:not(.no-image) > li > a > span.no-image' => 'padding-right: calc({{SIZE}}{{UNIT}}/2);',
                    'body:not(.rtl) {{WRAPPER}} ul.dm-products > li > a > img + div' => 'padding-left: calc({{SIZE}}{{UNIT}}/2);',
                    'body:not(.rtl) {{WRAPPER}} ul.dm-products:not(.no-image) > li > a > span.no-image + div' => 'padding-left: calc({{SIZE}}{{UNIT}}/2);',
                ],
                'default' => [
                    'size' => 20,
                ],
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'section_item_style',
            [
                'label' => __('Item', 'elementor-pro'),
                'tab' => Controls_Manager::TAB_STYLE,
                'show_label' => false,
            ]
        );

        $this->add_control(
            'row_gap',
            [
                'label' => __('Rows Gap', 'elementor-pro'),
                'type' => Controls_Manager::SLIDER,
                'range' => [
                    'px' => [
                        'max' => 50,
                    ],
                    'em' => [
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'size_units' => ['px', 'em'],
                'selectors' => [
                    '{{WRAPPER}} ul.dm-products > li:not(:last-child)' => 'margin-bottom: {{SIZE}}{{UNIT}};',
                ],
                'default' => [
                    'size' => 20,
                ],
            ]
        );

        $this->add_control(
            'vertical_align',
            [
                'label' => __('Vertical Align', 'elementor-pro'),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'top' => [
                        'title' => __('Top', 'elementor-pro'),
                        'icon' => 'eicon-v-align-top',
                    ],
                    'center' => [
                        'title' => __('Center', 'elementor-pro'),
                        'icon' => 'eicon-v-align-middle',
                    ],
                    'bottom' => [
                        'title' => __('Bottom', 'elementor-pro'),
                        'icon' => 'eicon-v-align-bottom',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ul.dm-products > li > a' => 'align-items: {{VALUE}};',
                ],
                'selectors_dictionary' => [
                    'top' => 'flex-start',
                    'bottom' => 'flex-end',
                ],
                'default' => 'top',
            ]
        );

        $this->end_controls_section();
    }

    public function render()
    {
        $settings = $this->get_settings_for_display();
        $filter = ncore_retrieve($settings, 'filter', 'user_has_product');
        $products = ncore_retrieve($settings, 'products', []);
        $products = array_values(array_filter($products, function ($product) use ($filter) {
            if ($product['visible'] != 'yes') {
                return false;
            }
            if ($filter == 'all' || Plugin::$instance->editor->is_edit_mode()) {
                return true;
            }
            if (ncore_isLoggedIn()) {
                static $accessible;
                if (!isset($accessible)) {
                    /** @var digimember_UserProductData $model */
                    $model = dm_api()->load->model('data/user_product');
                    $accessible = array_map(function ($obj) {
                        return $obj->id;
                    }, $model->getAccessableProductLinks(ncore_userId()));
                }
                if ($filter == 'user_has_product') {
                    return in_array($product['id'], $accessible);
                } else {
                    return !in_array($product['id'], $accessible);
                }
            } else {
                return ($filter != 'user_has_product');
            }
        }));
        $no_image = !count(array_filter($products, function ($product) {
            return (bool)$product['image']['url'];
        }));
        if (count($products)) {
            ?>
            <ul class="dm-products<?php echo $no_image ? ' no-image' : ''; ?>">
                <?php
                foreach ($products as $product) {
                    $url = $product['link_enabled'] == 'yes' ? $this->getProductUrl($product['id']) : $product['link_custom']['url'];
                    ?>
                    <li>
                        <a href="<?php echo $url; ?>">
                            <?php
                            if ($product['image']['url']) {
                                echo $this->render_image($product, $settings);
                            } else {
                                ?>
                                <span class="no-image"></span>
                                <?php
                            }
                            ?>
                            <div>
                                <span>
                                    <?php echo $product['name']; ?>
                                </span>
                                <p>
                                    <?php echo $product['description']; ?>
                                </p>
                            </div>
                        </a>
                    </li>
                    <?php
                }
                ?>
            </ul>
            <?php
        } else {
            echo $settings['empty_list'];
        }
    }

    protected function content_template()
    {
        ?>
        <#
        var urls = <?php echo json_encode($this->getProductUrls()); ?>;
        var products = settings.products.filter(product => product.visible === 'yes');
        var no_image = products.filter(product => product.image.url !== '') === 0 ? ' no-image' : '';
        #>
        <ul class="dm-products{{ no_image }}">
            <#
            for (var i in products) {
            var product = products[i];
            var url = product.link_enabled === 'yes' ? urls[products.id] : product.link_custom.url;
            #>
            <li>
                <a href="{{ url }}">
                    <#
                    if ( product.image && product.image.id ) {

                    var image = {
                    id: product.image.id,
                    url: product.image.url,
                    size: settings.image_size_size,
                    dimension: settings.image_size_custom_dimension,
                    model: view.getEditModel()
                    };

                    var image_url = elementor.imagesManager.getImageUrl( image );

                    if ( image_url ) {
                    #>
                    <img src="{{ image_url }}" alt="{{ product.name }}"/>
                    <#
                    }
                    }
                    else {
                    #>
                    <span class="no-image"></span>
                    <#
                    }
                    #>
                    <div>
                            <span>
                                {{ product.name }}
                            </span>
                        <p>
                            {{ product.description }}
                        </p>
                    </div>
                </a>
            </li>
            <#
            }
            #>
        </ul>
        <?php
    }

    /**
     * @return string
     */
    private function render_image($item, $instance)
    {
        $image_id = $item['image']['id'];
        $image_size = $instance['image_size_size'];
        if ('custom' === $image_size) {
            $image_src = Group_Control_Image_Size::get_attachment_image_src($image_id, 'image_size', $instance);
        } else {
            $image_src = wp_get_attachment_image_src($image_id, $image_size);
            $image_src = $image_src[0];
        }

        return sprintf('<img src="%s" alt="%s" />', $image_src, $item['name']);
    }

    /**
     * @return array
     */
    private function getProductUrls()
    {
        static $urls;
        if (!isset($urls)) {
            /** @var digimember_ProductData $model */
            $model = dm_api()->load->model('data/product');
            $urls = $model->getProductUrls();
        }
        return $urls;
    }

    /**
     * @return string|null
     */
    private function getProductUrl($id)
    {
        return ncore_retrieve($this->getProductUrls(), $id);
    }
}
