<?php

/**
 * @param mixed $transient
 *
 * @return mixed
 */
function digimember_elementor_update($transient)
{
    if (empty($transient) || !is_object($transient)) {
        return $transient;
    }

    $filePath = WP_CONTENT_DIR . '/plugins/digimember-elementor/plugin.php';
    if (!file_exists($filePath)) {
        return $transient;
    }
    $pluginData = get_plugin_data($filePath);
    if (!array_key_exists('Version', $pluginData)) {
        return $transient;
    }
    $pluginVersion = $pluginData['Version'];

    $update = digimember_elementor_fetchUpdateFromServer();
    if (is_array($update) && array_key_exists('latestVersion', $update) && array_key_exists('url', $update) && array_key_exists('slug', $update) && $update['latestVersion'] != null) {
        if (version_compare($update['latestVersion'], $pluginVersion, '>')) {
            $updateInfo = new stdClass();

            $pluginFile = $update['slug'] . '/plugin.php';
            $updateInfo->new_version = $update['latestVersion'];
            $updateInfo->slug = $update['slug'];
            $updateInfo->plugin = $pluginFile;
            $updateInfo->url = 'https://digimember.de';
            $updateInfo->package = $update['url'];
//            $updateInfo->icons = ['default' => $flytools->icon . '?v=' . $updateInfo->new_version];
            $updateInfo->tested = get_bloginfo('version');

            $transient->response[$pluginFile] = (object)(array)$updateInfo;
        }
    }

    return $transient;
}

add_filter('pre_set_site_transient_update_plugins', 'digimember_elementor_update');

/**
 * @return null|array
 */
function digimember_elementor_fetchUpdateFromServer()
{
    $query = '
        {
            pluginVersions {
                slug
                latestVersion
                url
            }
        }
    ';

    $url = 'https://api.flytools.de/graphql/public';

    $post = wp_remote_post($url, ['body' => ['query' => $query]]);
    $response = json_decode(wp_remote_retrieve_body($post), true);
    if (is_array($response)) {
        if (array_key_exists('data', $response) && array_key_exists('pluginVersions', $response['data'])) {
            foreach ($response['data']['pluginVersions'] as $pluginVersion) {
                if (array_key_exists('slug', $pluginVersion) && array_key_exists('latestVersion', $pluginVersion) && array_key_exists('url', $pluginVersion) && $pluginVersion['slug'] == 'digimember-elementor') {
                    return $pluginVersion;
                }
            }
        }
    }

    return null;
}
